from __future__ import annotations

from collections import deque
from copy import deepcopy
from dataclasses import dataclass
from datetime import date
from typing import Any, Dict, Sequence, List, Optional, Tuple, Type, Union

from anchorpoint import TextQuoteSelector, TextPositionSelector
from anchorpoint.utils.ranges import RangeSet
from anchorpoint.schemas import SelectorSchema, TextPositionSetFactory
from anchorpoint.textselectors import TextPositionSet
from anchorpoint.textsequences import TextPassage, TextSequence

# Path parts known to indicate the level of law they refer to.
KNOWN_CONSTITUTIONS = ["const"]
KNOWN_STATUTE_CODES = ["acts", "usc"]
KNOWN_REGULATIONS = ["cfr"]


RawSelector = Union[str, Dict[str, str]]
RawEnactment = Dict[str, Union[Any, str, List[RawSelector]]]


@dataclass
class CrossReference:
    """
    :param target_uri:
        the path to the target provision from the document root.

    :param target_url:
        the URL to fetch the target provision from an API.

    :param reference_text:
        The text in the cititng provision that represents the cross-reference.
        Generally, this text identifies the target provision.

    :param target_node:
        an identifier for the target URI in the API.
    """

    target_uri: str
    target_url: str
    reference_text: str
    target_node: Optional[int] = None

    def __repr__(self):
        return f'CrossReference(target_uri="{self.target_uri}", reference_text="{self.reference_text}")'


@dataclass
class CitingProvisionLocation:
    heading: str
    node: str
    start_date: date

    def __repr__(self):
        result = f"({self.node} {self.start_date})"
        return result

    def __gt__(self, other: CitingProvisionLocation):
        if self.start_date != other.start_date:
            return self.start_date > other.start_date
        return self.node > other.node


class InboundReference:
    def __init__(
        self,
        content: str,
        reference_text: str,
        target_uri: str,
        locations: List[CitingProvisionLocation],
    ) -> None:
        self.content = content
        self.reference_text = reference_text
        self.target_uri = target_uri
        self.locations = locations

    def __repr__(self):
        result = f"InboundReference to {self.target_uri}, from {max(self.locations)}"
        if len(self.locations) > 1:
            result = f"{result} and {len(self.locations) - 1} other locations"
        return result


class TextVersion:
    def __init__(
        self, content: str, url: Optional[str] = None, id: Optional[int] = None,
    ):
        if not content:
            raise ValueError("TextVersion should not be created with no content.")
        self.content = content
        self.url = url
        self.id = id


class BaseEnactment:
    """
    :param node:
        identifier for the site where the provision is codified

    :param heading:
        full heading of the provision

    :param content:
        full text content at this node, even if not all of it is cited

    :param start_date:
        date when the text was enacted at the cited location

    :param end_date:
        date when the text was removed from the cited location

    :param selection:
        identifier for the parts of the provision being cited

    :param anchors:
        a list of selectors representing the part of some other document
        (e.g. an Opinion) that references the Enactment. Unlike the selection
        field, it doesn't reference part of the Enactment itself. For use as
        a temporary place to store the anchors until they can be moved over
        to the citing document.

    :param name:
        a user-assigned label for the object
    """

    def __init__(
        self,
        node: str,
        heading: str,
        start_date: date,
        text_version: Optional[TextVersion] = None,
        content: Optional[str] = None,
        end_date: Optional[date] = None,
        anchors: Union[List[TextPositionSelector], List[TextQuoteSelector]] = None,
        citations: List[CrossReference] = None,
        name: str = "",
        *args,
        **kwargs,
    ):
        self.node = node
        if text_version:
            self.text_version = text_version
        elif content:
            self.text_version = TextVersion(content=content)
        else:
            self.text_version = None

        self._heading = heading
        self._start_date = start_date
        self._end_date = end_date
        self.anchors = anchors or []
        self._cross_references = citations or []
        self.name = name

    @property
    def source(self):
        return self.node

    @property
    def heading(self):
        return self._heading

    @property
    def content(self) -> str:
        if not self.text_version:
            return ""
        return self.text_version.content

    @property
    def start_date(self):
        return self._start_date

    @property
    def end_date(self):
        return self._end_date

    @property
    def children(self):
        return self._children

    @property
    def selection(self):
        return self._selection

    @property
    def text(self):
        return self.content

    @property
    def sovereign(self):
        identifier_parts = self.node.split("/")
        return identifier_parts[1]

    @property
    def jurisdiction(self):
        return self.sovereign

    @property
    def code(self):
        identifier_parts = self.node.split("/")
        if len(identifier_parts) < 3:
            return None
        return identifier_parts[2]

    @property
    def level(self):
        codename = self.code
        if codename in KNOWN_STATUTE_CODES:
            return "statute"
        if codename in KNOWN_CONSTITUTIONS:
            return "constitution"
        if codename in KNOWN_REGULATIONS:
            return "regulation"

    @property
    def padded_length(self):
        """Get length of self's content plus one character for space before next section."""
        if self.content:
            return len(self.content) + 1
        return 0

    def __str__(self):
        text_sequence = self.text_sequence()
        return f'"{text_sequence}" ({self.node} {self.start_date})'

    def __repr__(self):
        return f"{self.__class__.__name__}(source={self.source}, start_date={self.start_date}, selection={self.selection})"

    def cross_references(self) -> List[CrossReference]:
        """Return all cross-references from this node and subnodes."""
        result = self._cross_references[:]
        for child in self.children:
            result += child.cross_references()
        return result

    def selected_text(self) -> str:
        """
        Return this provision's text that is within the ranges described by self.selection.

        Based on creating an :class:`anchorpoint.textsequences.TextSequence` from this Enactment's
        text content and the ranges in its selection attribute.
        """
        text_sequence = self.text_sequence()
        return str(text_sequence)

    def get_passage(
        self,
        selection: Union[
            bool,
            str,
            TextPositionSelector,
            TextQuoteSelector,
            Sequence[TextQuoteSelector],
        ],
    ) -> str:

        position_set = self.convert_selection_to_set(selection)
        passage = position_set.as_string(text=self.text)
        return passage

    def convert_selection_to_set(
        self,
        selection: Union[
            bool,
            str,
            TextPositionSelector,
            TextPositionSet,
            TextQuoteSelector,
            Sequence[TextQuoteSelector],
        ],
    ) -> TextPositionSet:
        if selection is True:
            return TextPositionSet(
                TextPositionSelector(0, len(self.content) + 1, include_end=False)
            )
        factory = TextPositionSetFactory(self.text)
        position_set = factory.from_selection(selection)
        return position_set

    def convert_quotes_to_position(
        self, quotes: Sequence[TextQuoteSelector]
    ) -> TextPositionSet:
        factory = TextPositionSetFactory(passage=self.text)
        positions = factory.from_quote_selectors(quotes)
        return positions

    def select_from_text_positions_without_nesting(
        self, selections: Union[List[TextPositionSelector], RangeSet]
    ) -> TextPositionSet:
        r"""
        Move selectors from `selection` to `self._selection` and return any that can't be used.

        Replaces any preexisting _selection attribute on this Enactment object.

        :param selection:
            A TextPositionSet of TextPositionSelectors to apply to this Enactment.

        :returns:
            Any unused selections (beyond the end of the node content)
        """
        self._selection: List[TextPositionSelector] = []

        if isinstance(selections, RangeSet):
            selections = selections.ranges()
        selections = deque(selections)

        while selections and selections[0].start < len(self.content):
            current = selections.popleft()
            if current.end < len(self.content):
                self._selection.append(current)
            else:
                self._selection.append(
                    TextPositionSelector(start=current.start, end=len(self.content))
                )
                if current.end > self.padded_length:
                    selections.appendleft(
                        TextPositionSelector(start=self.padded_length, end=current.end)
                    )
        self._selection = TextPositionSet(self._selection)
        self._selection = self._selection.add_margin(text=self.content, margin_width=4)
        return TextPositionSet(selections)

    def select_without_children(
        self,
        selection: Union[
            bool,
            str,
            TextPositionSelector,
            TextPositionSet,
            TextQuoteSelector,
            Sequence[TextQuoteSelector],
        ],
    ) -> None:

        if not isinstance(selection, TextPositionSet):
            selection = self.convert_selection_to_set(selection)
        unused_selectors = self.select_from_text_positions_without_nesting(selection)
        self.raise_error_for_extra_selector(unused_selectors)

    def select(
        self,
        selection: Union[
            bool,
            TextPositionSelector,
            TextPositionSet,
            TextQuoteSelector,
            Sequence[TextQuoteSelector],
        ],
    ) -> None:
        self.select_without_children(selection)

    def text_sequence(self, include_nones: bool = True) -> TextSequence:
        """
        List the phrases in the Enactment selected by TextPositionSelectors.

        :param include_nones:
            Whether the list of phrases should include `None` to indicate a block of
            unselected text
        """
        return self.selection.as_text_sequence(
            text=self.content, include_nones=include_nones
        )

    def raise_error_for_extra_selector(
        self, unused_selectors: List[TextPositionSelector]
    ) -> None:
        for selector in unused_selectors:
            if selector.start > len(self.content) + 1:
                raise ValueError(f'Selector "{selector}" was not used.')


class LinkedEnactment(BaseEnactment):
    """
    One or more passages of legislative text, selected from within a cited location.

    :param children:
        URLs of other nodes nested within this one
    """

    def __init__(
        self,
        children: List[str] = None,
        selection: Union[bool, List[TextPositionSelector]] = True,
        *args,
        **kwargs,
    ):

        if not children:
            self._children = []
        else:
            self._children = children
        super().__init__(*args, **kwargs)

        if selection:
            self.select_without_children(selection)
        else:
            self._selection = TextPositionSet()

    def select_all(
        self,
        selection: Union[
            bool,
            TextPositionSelector,
            TextPositionSet,
            TextQuoteSelector,
            Sequence[TextQuoteSelector],
        ] = True,
    ) -> None:
        self.select_without_children(selection)


class Enactment(BaseEnactment):
    """
    One or more passages of legislative text, selected from within a cited location.

    :param children:
        other Enactments nested under this one's node

    """

    def __init__(
        self,
        children: List[Enactment] = None,
        selection: Union[bool, List[TextPositionSelector]] = True,
        *args,
        **kwargs,
    ):
        if not children:
            self._children = []
        else:
            self._children = children
        super().__init__(*args, **kwargs)
        self._selection = TextPositionSet()
        if selection:
            self.select_more(selection)

    @property
    def text(self):
        """Get all text including subnodes, regardless of which text is "selected"."""
        text_parts = [self.content]
        for child in self.children:
            if child.text:
                text_parts.append(child.text)
        joined = " ".join(text_parts)
        return joined.strip()

    def _tree_selection(
        self, selector_set: TextPositionSet, tree_length: int
    ) -> Tuple[TextPositionSet, int]:
        selectors_at_node = self.selection
        selectors_at_node_with_offset = selectors_at_node + tree_length
        new_tree_length = tree_length + self.padded_length
        new_selector_set = selector_set + selectors_at_node_with_offset

        for child in self.children:
            new_selector_set, new_tree_length = child._tree_selection(
                selector_set=new_selector_set, tree_length=new_tree_length
            )

        return new_selector_set, new_tree_length

    def tree_selection(self) -> TextPositionSet:
        """Return set of selectors for selected text in this provision and its children."""
        new_selector_set, new_tree_length = self._tree_selection(
            selector_set=TextPositionSet(), tree_length=0
        )
        return new_selector_set

    def select_more_text_at_current_node(
        self, added_selection: TextPositionSet
    ) -> None:
        """Select more text at this Enactment's node, not in child nodes."""
        new_selection = self.selection + added_selection
        self._selection = new_selection

    def select_more_text_in_current_branch(
        self, added_selection: TextPositionSet
    ) -> TextPositionSet:
        """Select more text within this Enactment's tree_selection, including child nodes."""
        new_selection = self.tree_selection() + added_selection
        unused_selectors = self.select_from_text_positions(new_selection)
        return unused_selectors

    def _update_text_at_included_node(self, other: Enactment) -> Tuple[bool, bool]:
        """Recursively search child nodes for one that can be updated by `other`."""
        if self.node == other.node:
            found_node = True
            if self.text == other.text:
                self.select_more_text_in_current_branch(other.selection)
                updated_selection = True
            else:
                try:
                    self.select_more_text_from_changed_version(other)
                    updated_selection = True
                except ValueError:
                    updated_selection = False
            return found_node, updated_selection
        for child in self.children:
            if other.node.startswith(child.node):
                return child._update_text_at_included_node(other)
        return False, False

    def _add_enactment_at_included_node(self, other: Enactment) -> Enactment:
        """Add a selection of text at the same citation or at a child of self's citation."""
        if self >= other:
            return self

        copy_of_self = deepcopy(self)
        found_node, updated_selection = copy_of_self._update_text_at_included_node(
            other
        )
        if not found_node:
            raise ValueError(
                f"Unable to find node {other.node} (dated {other.start_date}) "
                f"among the descendants of node {self.node} (dated {self.start_date})."
            )
        if not updated_selection:
            raise ValueError(
                f'Unable to find the selected text "{other.selected_text()}" '
                f"(dated {other.start_date}) "
                f"at the citation {other.node} (dated {self.start_date})."
            )
        return copy_of_self

    def __add__(self, other: Enactment):

        if not isinstance(other, self.__class__):
            copy_of_self = deepcopy(self)
            copy_of_self.select_more(other)
            return copy_of_self

        if other.node.startswith(self.node):
            return self._add_enactment_at_included_node(other)
        elif self.node.startswith(other.node):
            return other._add_enactment_at_included_node(self)

        raise ValueError(
            "Can't add selected text from two different Enactments "
            "when neither is a descendant of the other."
        )

    def select_from_text_positions(self, selection: TextPositionSet) -> TextPositionSet:
        """Select text using position selectors and return any unused position selectors."""
        selections = self.select_from_text_positions_without_nesting(selection)

        selections = [selection - self.padded_length for selection in selections]
        for child in self.children:
            selections = child.select_from_text_positions(TextPositionSet(selections))
        return selections

    def select_all(self) -> None:
        self._selection = TextPositionSet(TextPositionSelector(0, len(self.content)))
        for child in self._children:
            child.select_all()

    def select_none(self) -> None:
        self._selection = TextPositionSet()
        for child in self._children:
            child.select_none()

    def select_more_text_from_changed_version(self, other: Enactment) -> None:
        """
        Select more text from a different text version at the same citation path.

        :param other:
            An :class:`Enactment` representing different text enacted at a
            different time, at the same `node` (or USLM path citation) as self.
            This Element's :attr:`~Enactment.node` attribute must be the same
            string as self's node attribute. It's not sufficient for `other`
            to have an Enactment listed in its :attr:`~Enactment._children`
            attribute with the same node attribute,
            or for `other` to have the same node attribute as an ancestor of self.
        """
        incoming_quote_selectors = [
            selector.as_quote_selector(other.text)
            for selector in other.tree_selection()
        ]
        incoming_position_selectors = []
        for quote_selector in incoming_quote_selectors:
            position = quote_selector.as_position(self.text)
            if position:
                incoming_position_selectors.append(position)
            else:
                raise ValueError(
                    f"Incoming text selection {quote_selector}  "
                    f"cannot be found in the provision text."
                )
            if not quote_selector.is_unique_in(self.text):
                raise ValueError(
                    f"Incoming text selection {quote_selector} cannot be placed because it "
                    f"is not unique in the provision text."
                )
        self.select_more_text_in_current_branch(
            TextPositionSet(incoming_position_selectors)
        )

    def select_more(
        self,
        selection: Union[
            str,
            TextPositionSelector,
            TextPositionSet,
            TextQuoteSelector,
            Sequence[TextQuoteSelector],
        ],
    ) -> None:
        """
        Select text, in addition to any previous selection.
        """
        if not isinstance(selection, TextPositionSet):
            try:
                selection = self.convert_selection_to_set(selection)
            except ValueError:
                raise TypeError(
                    f"Cannot select more Enactment text with type {type(selection)}."
                )

        # Ignore child nodes if selector was passed in without an end
        if any(selector.end > 99999 for selector in selection):
            self.select_without_children(True)
        else:
            unused_selectors = self.select_more_text_in_current_branch(selection)
            self.raise_error_for_extra_selector(unused_selectors)

    def select(
        self,
        selection: Union[
            bool,
            str,
            TextPositionSelector,
            TextPositionSet,
            TextQuoteSelector,
            Sequence[TextQuoteSelector],
        ] = True,
    ) -> None:
        """
        Select text, clearing any previous selection.

        If the selection includes no selectors for child Enactments,
        then any selected passages for the child Enactments will be
        cleared.
        """
        if selection is True:
            self.select_all()
        elif selection is False or selection is None:
            self.select_none()
        else:
            if not isinstance(selection, TextPositionSet):
                selection = self.convert_selection_to_set(selection)
            unused_selectors = self.select_from_text_positions(selection)
            self.raise_error_for_extra_selector(unused_selectors)

    def text_sequence(self, include_nones: bool = True) -> TextSequence:
        """
        List the phrases in the Enactment selected by TextPositionSelectors.

        :param include_nones:
            Whether the list of phrases should include `None` to indicate a block of
            unselected text
        """
        selected = super().text_sequence(include_nones=include_nones)
        for child in self.children:
            if selected:
                selected = selected + child.text_sequence(include_nones=include_nones)
            else:
                selected = child.text_sequence(include_nones=include_nones)

        return selected

    def means(self, other: Enactment) -> bool:
        r"""
        Find whether meaning of ``self`` is equivalent to that of ``other``.

        ``Self`` must be neither broader nor narrower than ``other`` to return True.

        :returns:
            whether ``self`` and ``other`` represent the same text
            issued by the same sovereign in the same level of
            :class:`Enactment`\.
        """
        if not isinstance(other, self.__class__):
            raise TypeError(
                f"Cannot compare {self.__class__.__name__} and {other.__class__.__name__} for same meaning."
            )
        self_selected_passages = self.text_sequence()
        other_selected_passages = other.text_sequence()
        return self_selected_passages.means(other_selected_passages)

    def implies(self, other: Enactment) -> bool:
        """Test whether ``self`` has all the text passages of ``other``."""
        if not isinstance(other, self.__class__):
            raise TypeError(
                f"Cannot compare {self.__class__.__name__} and {other.__class__.__name__} for implication."
            )
        self_selected_passages = self.text_sequence(include_nones=False)
        other_selected_passages = other.text_sequence(include_nones=False)
        return self_selected_passages >= other_selected_passages

    def __ge__(self, other: Enactment) -> bool:
        """
        Test whether ``self`` implies ``other``.

        :returns:
            Whether ``self`` contains at least all the same text as ``other``.
        """
        return self.implies(other)

    def __gt__(self, other) -> bool:
        """Test whether ``self`` implies ``other`` without having same meaning."""
        if self.means(other):
            return False
        return self.implies(other)


def consolidate_enactments(enactments: List[Enactment]) -> List[Enactment]:
    r"""
    Consolidate any overlapping :class:`Enactment`\s in a :class:`list`.

    :param enactments:
        a list of :class:`Enactment`\s that may include overlapping
        legislative text within the same section

    :returns:
        a list of :class:`Enactment`\s without overlapping text
    """
    consolidated: List[Enactment] = []
    while enactments:
        match_made = False
        left = enactments.pop()
        for right in enactments:
            try:
                combined = left + right
                enactments.remove(right)
                enactments.append(combined)
                match_made = True
                break
            except ValueError:
                pass
        if match_made is False:
            consolidated.append(left)
    return consolidated



import datetime
from os import name
from typing import Any, Dict, List, Optional, Union

from marshmallow import ValidationError
from marshmallow.fields import Raw
import requests
from requests import status_codes

from legislice.enactments import (
    Enactment,
    CrossReference,
    CitingProvisionLocation,
    InboundReference,
)
from legislice.name_index import EnactmentIndex
from legislice.schemas import (
    ExpandableSchema,
    InboundReferenceSchema,
    get_schema_for_node,
    enactment_needs_api_update,
)

RawEnactment = Dict[str, Any]


class LegislicePathError(Exception):
    pass


class LegisliceDateError(Exception):
    pass


class LegisliceTokenError(Exception):
    pass


def normalize_path(path: str) -> str:
    return "/" + path.strip("/")


class Client:
    def __init__(
        self, api_token: str = "", api_root: str = "https://authorityspoke.com/api/v1"
    ):
        """Create download client with an API token and an API address."""
        self.api_root = api_root

        if api_token.startswith("Token "):
            api_token = api_token.split("Token ")[1]
        self.api_token = api_token

    def fetch(
        self,
        query: Union[str, CitingProvisionLocation, CrossReference, InboundReference],
        date: Union[datetime.date, str] = "",
    ) -> RawEnactment:
        """
        Download legislative provision from string identifier or cross-reference.

        :param query:
            A cross-reference to the desired legislative provision, or a path to the
            desired legislation section using the United States Legislation Markup
            tree-like citation format.

        :param date:
            The date of the desired version of the provision to be downloaded. This is
            not needed if a :class:`~legislice.enactments.CrossReference` passed to the
            ``query`` param specifies a date. If no date is provided, the API will use the
            most recent date.
        """
        if isinstance(query, CrossReference):
            return self.fetch_cross_reference(query=query, date=date)
        elif isinstance(query, CitingProvisionLocation):
            return self.fetch_citing_provision(query=query)
        elif isinstance(query, InboundReference):
            return self.fetch_inbound_reference(query=query)
        return self.fetch_uri(query=query, date=date)

    def fetch_citing_provision(self, query: CitingProvisionLocation) -> RawEnactment:
        """
        Download legislative provision as Enactment from CitingProvisionLocation.

        CitingProvisionLocations are found in the `locations` attribute of the `InboundReference`
        objects obtained when using the `citations_to` method.
        """
        return self.fetch_uri(query=query.node, date=query.start_date)

    def fetch_cross_reference(
        self, query: CrossReference, date: Union[datetime.date, str] = ""
    ) -> RawEnactment:
        """
        Download legislative provision from cross-reference.

        :param query:
            A cross-reference to the desired legislative provision. Found by calling the
            :meth:`~legislice.enactments.Enactment.cross_references` method on an
            :class:`~legislice.enactments.Enactment` that contains one or more citations
            to other provisions.

        :param date:
            The date of the desired version of the provision to be downloaded. This is
            not needed if the :class:`~legislice.enactments.CrossReference` passed to the
            ``query`` param specifies a date. If no date is provided, the API will use the
            most recent date.
        """
        if isinstance(date, datetime.date):
            date = date.isoformat()

        target = query.target_url

        if date:
            if "@" in target:
                if not target.endswith(date):
                    raise ValueError(
                        f"Date param {date} does not match date in URL {target}"
                    )
            else:
                target = f"{target}@{date}"

        if not target.startswith(self.api_root):
            raise ValueError(
                f'target_url of cross-reference, "{target}", does not start with Client\'s api_root, "{self.api_root}"'
            )
        return self._fetch_from_url(url=target)

    def fetch_inbound_reference(self, query: InboundReference) -> RawEnactment:
        """
        Download legislative provision from InboundReference.

        :param query:
            An InboundReference identifying a provision containing a citation.

        :returns:
            An Enactment representing the provision containing the citation (not the cited provision).

        If the InboundReference has been enacted more than once, the latest one will be chosen.
        """
        most_recent = max(query.locations)
        return self.fetch_citing_provision(query=most_recent)

    def fetch_uri(
        self, query: str, date: Union[datetime.date, str] = ""
    ) -> RawEnactment:
        """
        Fetches data about legislation at specified path and date from Client's assigned API root.

        :param query:
            A path to the desired legislation section using the United States Legislation Markup
            tree-like citation format. Examples: "/us/const/amendment/IV", "/us/usc/t17/s103"

        :param date:
            A date when the desired version of the legislation was in effect. This does not need to
            be the "effective date" or the first date when the version was in effect. However, if
            you select a date when two versions of the provision were in effect at the same time,
            you will be given the version that became effective later.
        """
        query_with_root = self.api_root + normalize_path(query)

        if isinstance(date, datetime.date):
            date = date.isoformat()
        if date:
            query_with_root = f"{query_with_root}@{date}"

        return self._fetch_from_url(url=query_with_root)

    def uri_from_query(self, target: Union[str, Enactment, CrossReference]) -> str:
        if isinstance(target, Enactment):
            return target.node
        elif isinstance(target, CrossReference):
            return target.target_uri
        return target

    def fetch_citations_to(
        self, target: Union[str, Enactment, CrossReference]
    ) -> List[Dict]:
        """
        Query API for citations to a given target node, withoout loading them as InboundCitations.

        :param target:
            a string URI for the cited node, an Enactment at the cited node, or
            a CrossReference to the cited node

        :returns:
            a list of dicts representing citations to the cited node
        """
        uri = self.uri_from_query(target)
        query_with_root = self.api_root + "/citations_to" + uri
        api_response = self._fetch_from_url(query_with_root)
        return api_response["results"]

    def citations_to(
        self, target: Union[str, Enactment, CrossReference]
    ) -> List[InboundReference]:
        r"""
        Load an InboundReference object for each provision citing the target USLM provision URI.

        :param target:
            a string URI for the cited node, an Enactment at the cited node, or
            a CrossReference to the cited node

        :returns:
            a list of InboundReferences to the cited node
        """
        target_uri = self.uri_from_query(target)
        json_citations = self.fetch_citations_to(target_uri)
        for entry in json_citations:
            entry["target_uri"] = target_uri
        schema = InboundReferenceSchema(many=True)
        citations = schema.load(json_citations)
        return citations

    def read_from_json(self, data: RawEnactment) -> Enactment:
        r"""
        Create a new :class:`Enactment` object using imported JSON data.

        If fields are missing from the JSON, they will be fetched using the API key.
        """
        data_has_selection_field = "selection" in data.keys()

        if enactment_needs_api_update(data):
            data = self.update_enactment_from_api(data)
        schema_class = get_schema_for_node(data["node"])
        schema = schema_class()
        enactment = schema.load(data)

        if not enactment.selected_text() and not data_has_selection_field:
            enactment.select_all()
        return enactment

    def read(
        self, query: Union[str, CrossReference], date: Union[datetime.date, str] = "",
    ) -> Enactment:
        """
        Fetches data from Client's assigned API root and builds Enactment or LinkedEnactment.

        All text is selected by default.

        :param path:
            A path to the desired legislation section using the United States Legislation Markup
            tree-like citation format. Examples: "/us/const/amendment/IV", "/us/usc/t17/s103"

        :param date:
            A date when the desired version of the legislation was in effect. This does not need to
            be the "effective date" or the first date when the version was in effect. However, if
            you select a date when two versions of the provision were in effect at the same time,
            you will be given the version that became effective later.
        """
        raw_enactment = self.fetch(query=query, date=date)
        enactment = self.read_from_json(raw_enactment)
        enactment.select_all()
        return enactment

    def update_enactment_from_api(self, data: RawEnactment) -> RawEnactment:
        """
        Use API to fill in missing fields in a dict representing an :class:`~legislice.enactments.Enactment`.

        Useful when the dict has missing data because it was created by a user.
        """

        data_from_api = self.fetch(query=data["node"], date=data.get("start_date"))
        new_data = {**data, **data_from_api}
        return new_data

    def update_entries_in_enactment_index(
        self, enactment_index: EnactmentIndex
    ) -> EnactmentIndex:
        """Fill in missing fields in every entry in an :class:`~legislice.name_index.EnactmentIndex`."""
        for key, value in enactment_index.items():
            if enactment_needs_api_update(value):
                enactment_index[key] = self.update_enactment_from_api(value)
        return enactment_index

    def _fetch_from_url(self, url: str) -> RawEnactment:
        headers = {}
        if self.api_token:
            headers["Authorization"] = f"Token {self.api_token}"

        response = requests.get(url, headers=headers)
        if response.status_code == 404:
            raise LegislicePathError(f"No enacted text found for query {url}")
        if response.status_code == 403:
            raise LegisliceTokenError(f"{response.json().get('detail')}")

        return response.json()



from copy import deepcopy
from datetime import date
from typing import Dict, List, Union

from anchorpoint.schemas import SelectorSchema
from marshmallow import Schema, fields, post_load, pre_load, EXCLUDE, ValidationError

from legislice.enactments import (
    Enactment,
    InboundReference,
    LinkedEnactment,
    RawEnactment,
    CrossReference,
    TextVersion,
    CitingProvisionLocation,
)
from legislice.name_index import EnactmentIndex


def enactment_needs_api_update(data: RawEnactment) -> bool:
    """Determine if JSON representation of Enactment needs to be supplemented from API."""
    if not data.get("node"):
        raise ValueError(
            '"data" must contain a "node" field '
            "with a citation path to a legislative provision, "
            'for example "/us/const/amendment/IV"'
        )
    if data.get("heading") is None or data.get("start_date") is None:
        return True
    if data.get("content") is None and data.get("text_version") is None:
        return False


class CitingProvisionLocationSchema(Schema):
    __model__ = CitingProvisionLocation

    heading = fields.Str()
    node = fields.Str()
    start_date = fields.Date()

    class Meta:
        unknown = EXCLUDE

    @post_load
    def make_object(self, data, **kwargs) -> CitingProvisionLocation:
        return self.__model__(**data)


class InboundReferenceSchema(Schema):
    __model__ = InboundReference

    content = fields.Str()
    reference_text = fields.Str()
    target_uri = fields.Str()
    locations = fields.Nested(CitingProvisionLocationSchema, many=True)

    class Meta:
        unknown = EXCLUDE

    @pre_load
    def format_data_to_load(
        self, data: Dict[str, Union[Dict[str, str], str]], **kwargs
    ) -> Dict:

        reference_text = ""
        for citation in data["citations"]:
            if citation["target_uri"] == data["target_uri"]:
                reference_text = citation["reference_text"]
        data["reference_text"] = reference_text

        return data

    @post_load
    def make_object(self, data, **kwargs) -> InboundReference:
        return self.__model__(**data)


class CrossReferenceSchema(Schema):
    __model__ = CrossReference

    target_uri = fields.Str(required=True)
    target_url = fields.Url(relative=False, required=True)
    reference_text = fields.Str(required=True)
    target_node = fields.Int(required=False)

    class Meta:
        unknown = EXCLUDE

    @post_load
    def make_object(self, data, **kwargs) -> CrossReference:
        return self.__model__(**data)


class ExpandableSchema(Schema):
    """Base schema for classes that can be cross-referenced by name in input JSON."""

    def get_indexed_enactment(self, data, **kwargs):
        """Replace data to load with any object with same name in "enactment_index"."""
        if isinstance(data, str):
            name_to_retrieve = data
        elif data.get("name") and enactment_needs_api_update(data):
            name_to_retrieve = data["name"]
        else:
            return data

        mentioned = self.context.get("enactment_index") or EnactmentIndex()
        return deepcopy(mentioned.get_by_name(name_to_retrieve))

    def wrap_single_element_in_list(self, data: Dict, many_element: str):
        """Make a specified field a list if it isn't already a list."""
        if data.get(many_element) is None:
            data[many_element] = []
        elif not isinstance(data[many_element], list):
            data[many_element] = [data[many_element]]
        return data

    @post_load
    def make_object(self, data, **kwargs):
        if data.get("selection"):
            data["selection"] = [item for item in data["selection"] if item is not None]

        return self.__model__(**data)


class TextVersionSchema(Schema):
    __model__ = TextVersion
    content = fields.Str(required=True)

    class Meta:
        unknown = EXCLUDE

    @post_load
    def make_object(self, data, **kwargs):
        return self.__model__(**data)


class LinkedEnactmentSchema(ExpandableSchema):
    """Schema for passages from legislation without the full text of child nodes."""

    __model__ = LinkedEnactment
    node = fields.Url(relative=True, required=True)
    heading = fields.Str(required=True)
    text_version = fields.Nested(TextVersionSchema, required=False, missing=None)
    start_date = fields.Date(required=True)
    end_date = fields.Date(missing=None)
    children = fields.List(fields.Url(relative=False))
    selection = fields.Nested(SelectorSchema, many=True, missing=list)
    anchors = fields.Nested(SelectorSchema, many=True, missing=list)
    citations = fields.Nested(CrossReferenceSchema, many=True, missing=list)

    class Meta:
        unknown = EXCLUDE

    def move_selector_fields(self, data: RawEnactment, **kwargs):
        """
        Nest fields used for :class:`SelectorSchema` model.

        If the fields are already nested, they need not to be moved.

        The fields can only be moved into a "selector" field with a dict
        value, not a "selectors" field with a list value.
        """

        selector_field_names = ["text", "exact", "prefix", "suffix", "start", "end"]
        new_selector = {}
        for name in selector_field_names:
            if data.get(name):
                new_selector[name] = data[name]
                del data[name]
        if new_selector:
            data["selection"].append(new_selector)
        return data

    def nest_content_in_textversion(self, data):
        if data.get("content"):
            if not data.get("text_version"):
                data["text_version"] = {}
            data["text_version"]["content"] = data["content"]
        data.pop("content", None)
        return data

    @pre_load
    def format_data_to_load(self, data, **kwargs):
        """Prepare Enactment to load."""
        data = self.get_indexed_enactment(data)
        data = self.nest_content_in_textversion(data)
        data = self.wrap_single_element_in_list(data, "selection")
        data = self.move_selector_fields(data)
        data = self.wrap_single_element_in_list(data, "anchors")
        return data


class EnactmentSchema(LinkedEnactmentSchema):
    """Schema for passages from legislation."""

    __model__ = Enactment
    children = fields.List(fields.Nested(lambda: EnactmentSchema()))

    class Meta:
        unknown = EXCLUDE


def get_schema_for_node(path: str):
    if path.count("/") < 4:
        return LinkedEnactmentSchema
    return EnactmentSchema



import setuptools

import legislice

with open("readme.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="legislice",
    version=legislice.__version__,
    author="Matt Carey",
    author_email="matt@authorityspoke.com",
    description="API client for fetching and comparing passages from legislation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mscarey/legislice",
    packages=setuptools.find_packages(
        exclude=["tests", "*.tests", "*.tests.*", "tests.*", "old"]
    ),
    install_requires=[
        "anchorpoint",
        "apispec",
        "marshmallow",
        "python-dotenv",
        "requests",
        "roman",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Legal Industry",
        "License :: Free To Use But Restricted",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Topic :: Sociology :: History",
    ],
    python_requires=">=3.7",
    include_package_data=True,
)



import datetime
import json
import os
from typing import Dict, Optional, Union

from legislice.download import (
    Client,
    LegislicePathError,
    LegisliceDateError,
    RawEnactment,
    normalize_path,
)
from legislice.enactments import CrossReference

# A dict indexing responses by iso-format date strings.
ResponsesByDate = Dict[str, Dict]
ResponsesByDateByPath = Dict[str, Dict[str, Dict]]


class JSONRepository(Client):
    """Repository for mocking API responses locally."""

    def __init__(self, responses: ResponsesByDateByPath):
        self.responses = responses

    def get_entry_closest_to_cited_path(self, path: str) -> Optional[ResponsesByDate]:
        path = normalize_path(path)
        if self.responses.get(path):
            return self.responses[path]
        branches_that_start_path = [
            entry for entry in self.responses.keys() if path.startswith(entry)
        ]
        if not branches_that_start_path:
            return None
        name_of_best_entry = max(branches_that_start_path, key=len)
        return self.responses[name_of_best_entry]

    def search_tree_for_path(
        self, path: str, branch: Dict
    ) -> Optional[ResponsesByDate]:
        path = normalize_path(path)
        if branch["node"] == path:
            return branch
        branches_that_start_path = [
            nested_node
            for nested_node in branch["children"]
            if path.startswith(nested_node["node"])
        ]
        if branches_that_start_path:
            return self.search_tree_for_path(
                path=path, branch=branches_that_start_path[0]
            )
        return None

    def fetch(
        self, query: Union[str, CrossReference], date: Union[datetime.date, str] = ""
    ) -> RawEnactment:
        """
        Fetches data about legislation at specified path and date from Client's assigned API root.

        :param path:
            A path to the desired legislation section using the United States Legislation Markup
            tree-like citation format. Examples: "/us/const/amendment/IV", "/us/usc/t17/s103"

        :param date:
            A date when the desired version of the legislation was in effect. This does not need to
            be the "effective date" or the first date when the version was in effect. However, if
            you select a date when two versions of the provision were in effect at the same time,
            you will be given the version that became effective later.
        """
        responses = self.get_entry_closest_to_cited_path(query)
        if not responses:
            raise LegislicePathError(f"No enacted text found for query {query}")

        if isinstance(date, datetime.date):
            date = date.isoformat()

        if not date:
            selected_date = max(responses.keys())
        else:
            versions_not_later_than_query = [
                version_date
                for version_date in responses.keys()
                if version_date <= date
            ]
            if not versions_not_later_than_query:
                raise LegisliceDateError(
                    f"No enacted text found for query {query} after date {date}"
                )
            selected_date = max(versions_not_later_than_query)

        selected_version = responses[selected_date]

        result = self.search_tree_for_path(path=query, branch=selected_version)
        if not result:
            raise LegislicePathError(
                f"No enacted text found for query {query} after date {date}"
            )
        return result


MOCK_USC_RESPONSES = {
    "/us/const/article/I/8/8": {
        "1788-09-13": {
            "heading": "Patents and copyrights.",
            "content": "To promote the Progress of Science and useful Arts, by securing for limited Times to Authors and Inventors the exclusive Right to their respective Writings and Discoveries;",
            "children": [],
            "end_date": None,
            "node": "/us/const/article/I/8/8",
            "start_date": "1788-09-13",
            "url": "https://authorityspoke.com/api/v1/us/const/article/I/8/8/",
            "parent": "https://authorityspoke.com/api/v1/us/const/article/I/8/",
        }
    },
    "/us/const/amendment/IV": {
        "1791-12-15": {
            "heading": "AMENDMENT IV.",
            "content": "The right of the people to be secure in their persons, houses, papers, and effects, against unreasonable searches and seizures, shall not be violated, and no Warrants shall issue, but upon probable cause, supported by Oath or affirmation, and particularly describing the place to be searched, and the persons or things to be seized.",
            "children": [],
            "end_date": None,
            "node": "/us/const/amendment/IV",
            "start_date": "1791-12-15",
            "url": "https://authorityspoke.com/api/v1/us/const/amendment/IV/",
            "parent": "https://authorityspoke.com/api/v1/us/const/amendment/",
        }
    },
    "/us/const/amendment/V": {
        "1791-12-15": {
            "heading": "AMENDMENT V.",
            "content": "No person shall be held to answer for a capital, or otherwise infamous crime, unless on a presentment or indictment of a Grand Jury, except in cases arising in the land or naval forces, or in the Militia, when in actual service in time of War or public danger; nor shall any person be subject for the same offence to be twice put in jeopardy of life or limb; nor shall be compelled in any Criminal Case to be a witness against himself; nor be deprived of life, liberty, or property, without due process of law; nor shall private property be taken for public use, without just compensation.",
            "children": [],
            "end_date": None,
            "node": "/us/const/amendment/V",
            "start_date": "1791-12-15",
            "url": "https://authorityspoke.com/api/v1/us/const/amendment/V/",
            "parent": "https://authorityspoke.com/api/v1/us/const/amendment/",
        }
    },
    "/us/const/amendment/XIV": {
        "1868-07-28": {
            "heading": "AMENDMENT XIV.",
            "content": "",
            "children": [
                {
                    "heading": "Citizenship: security and equal protection of citizens.",
                    "content": "All persons born or naturalized in the United States, and subject to the jurisdiction thereof, are citizens of the United States and of the State wherein they reside. No State shall make or enforce any law which shall abridge the privileges or immunities of citizens of the United States; nor shall any State deprive any person of life, liberty, or property, without due process of law; nor deny to any person within its jurisdiction the equal protection of the laws.",
                    "children": [],
                    "end_date": None,
                    "node": "/us/const/amendment/XIV/1",
                    "start_date": "1868-07-28",
                    "url": "https://authorityspoke.com/api/v1/us/const/amendment/XIV/1/",
                },
                {
                    "heading": "Apportionment of representation.",
                    "content": "Representatives shall be apportioned among the several States according to their respective numbers, counting the whole number of persons in each State, excluding Indians not taxed. But when the right to vote at any election for the choice of electors for President and Vice President of the United States, Representatives in Congress, the Executive and Judicial officers of a State, or the members of the Legislature thereof, is denied to any of the male inhabitants of such State, being twenty-one years of age, and citizens of the United States, or in any way abridged, except for participation in rebellion, or other crime, the basis of representation therein shall be reduced in the proportion which the number of such male citizens shall bear to the whole number of male citizens twenty-one years of age in such State.",
                    "children": [],
                    "end_date": None,
                    "node": "/us/const/amendment/XIV/2",
                    "start_date": "1868-07-28",
                    "url": "https://authorityspoke.com/api/v1/us/const/amendment/XIV/2/",
                },
                {
                    "heading": "Loyalty as a qualification of Senators and Representatives.",
                    "content": "No person shall be a Senator or Representative in Congress, or elector of President and Vice President, or hold any office, civil or military, under the United States, or under any State, who, having previously taken an oath, as a member of Congress, or as an officer of the United States, or as a member of any State legislature, or as an executive or judicial officer of any State, to support the Constitution of the United States, shall have engaged in insurrection or rebellion against the same, or given aid or comfort to the enemies thereof. But Congress may by a vote of two-thirds of each House, remove such disability.",
                    "children": [],
                    "end_date": None,
                    "node": "/us/const/amendment/XIV/3",
                    "start_date": "1868-07-28",
                    "url": "https://authorityspoke.com/api/v1/us/const/amendment/XIV/3/",
                },
                {
                    "heading": "Validity of the national debt, etc.",
                    "content": "The validity of the public debt of the United States, authorized by law, including debts incurred for payment of pensions and bounties for services in suppressing insurrection or rebellion, shall not be questioned. But neither the United States nor any State shall assume or pay any debt or obligation incurred in aid of insurrection or rebellion against the United States, or any claim for the loss or emancipation of any slave; but all such debts, obligations and claims shall be held illegal and void.",
                    "children": [],
                    "end_date": None,
                    "node": "/us/const/amendment/XIV/4",
                    "start_date": "1868-07-28",
                    "url": "https://authorityspoke.com/api/v1/us/const/amendment/XIV/4/",
                },
                {
                    "heading": "Enforcement of the 14th amendment.",
                    "content": "The Congress shall have power to enforce, by appropriate legislation, the provisions of this article.",
                    "children": [],
                    "end_date": None,
                    "node": "/us/const/amendment/XIV/5",
                    "start_date": "1868-07-28",
                    "url": "https://authorityspoke.com/api/v1/us/const/amendment/XIV/5/",
                },
            ],
            "end_date": None,
            "node": "/us/const/amendment/XIV",
            "start_date": "1868-07-28",
            "url": "https://authorityspoke.com/api/v1/us/const/amendment/XIV/",
            "parent": "https://authorityspoke.com/api/v1/us/const/amendment/",
        }
    },
    "/us/const/amendment/XV": {
        "1870-03-30": {
            "heading": "AMENDMENT XV.",
            "content": "",
            "children": [
                {
                    "heading": "Suffrage not to be abridged for race, color, etc.",
                    "content": "The right of citizens of the United States to vote shall not be denied or abridged by the United States or by any State on account of race, color, or previous condition of servitude.",
                    "children": [],
                    "end_date": None,
                    "node": "/us/const/amendment/XV/1",
                    "start_date": "1870-03-30",
                    "url": "https://authorityspoke.com/api/v1/us/const/amendment/XV/1/",
                },
                {
                    "heading": "Section 2.",
                    "content": "The Congress shall have power to enforce this article by appropriate legislation.",
                    "children": [],
                    "end_date": None,
                    "node": "/us/const/amendment/XV/2",
                    "start_date": "1870-03-30",
                    "url": "https://authorityspoke.com/api/v1/us/const/amendment/XV/2/",
                },
            ],
            "end_date": None,
            "node": "/us/const/amendment/XV",
            "start_date": "1870-03-30",
            "url": "https://authorityspoke.com/api/v1/us/const/amendment/XV/",
            "parent": "https://authorityspoke.com/api/v1/us/const/amendment/",
        }
    },
    "/us/usc/t17/s102/a": {
        "2013-07-18": {
            "heading": "",
            "content": "Copyright protection subsists, in accordance with this title, in original works of authorship fixed in any tangible medium of expression, now known or later developed, from which they can be perceived, reproduced, or otherwise communicated, either directly or with the aid of a machine or device. Works of authorship include the following categories:",
            "children": [
                {
                    "heading": "",
                    "content": "literary works;",
                    "children": [],
                    "end_date": None,
                    "node": "/us/usc/t17/s102/a/1",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/us/usc/t17/s102/a/1/",
                },
                {
                    "heading": "",
                    "content": "musical works, including any accompanying words;",
                    "children": [],
                    "end_date": None,
                    "node": "/us/usc/t17/s102/a/2",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/us/usc/t17/s102/a/2/",
                },
                {
                    "heading": "",
                    "content": "dramatic works, including any accompanying music;",
                    "children": [],
                    "end_date": None,
                    "node": "/us/usc/t17/s102/a/3",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/us/usc/t17/s102/a/3/",
                },
                {
                    "heading": "",
                    "content": "pantomimes and choreographic works;",
                    "children": [],
                    "end_date": None,
                    "node": "/us/usc/t17/s102/a/4",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/us/usc/t17/s102/a/4/",
                },
                {
                    "heading": "",
                    "content": "pictorial, graphic, and sculptural works;",
                    "children": [],
                    "end_date": None,
                    "node": "/us/usc/t17/s102/a/5",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/us/usc/t17/s102/a/5/",
                },
                {
                    "heading": "",
                    "content": "motion pictures and other audiovisual works;",
                    "children": [],
                    "end_date": None,
                    "node": "/us/usc/t17/s102/a/6",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/us/usc/t17/s102/a/6/",
                },
                {
                    "heading": "",
                    "content": "sound recordings; and",
                    "children": [],
                    "end_date": None,
                    "node": "/us/usc/t17/s102/a/7",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/us/usc/t17/s102/a/7/",
                },
                {
                    "heading": "",
                    "content": "architectural works.",
                    "children": [],
                    "end_date": None,
                    "node": "/us/usc/t17/s102/a/8",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/us/usc/t17/s102/a/8/",
                },
            ],
            "end_date": None,
            "node": "/us/usc/t17/s102/a",
            "start_date": "2013-07-18",
            "url": "https://authorityspoke.com/api/v1/us/usc/t17/s102/a/",
            "parent": "https://authorityspoke.com/api/v1/us/usc/t17/s102/",
        }
    },
    "/us/usc/t17/s102/b": {
        "2013-07-18": {
            "heading": "",
            "content": "In no case does copyright protection for an original work of authorship extend to any idea, procedure, process, system, method of operation, concept, principle, or discovery, regardless of the form in which it is described, explained, illustrated, or embodied in such work.",
            "children": [],
            "end_date": None,
            "node": "/us/usc/t17/s102/b",
            "start_date": "2013-07-18",
            "url": "https://authorityspoke.com/api/v1/us/usc/t17/s102/b/",
            "parent": "https://authorityspoke.com/api/v1/us/usc/t17/s102/",
        }
    },
    "/us/usc/t17/s103": {
        "2013-07-18": {
            "heading": "Subject matter of copyright: Compilations and derivative works",
            "content": "",
            "children": [
                {
                    "heading": "",
                    "content": "The subject matter of copyright as specified by section 102 includes compilations and derivative works, but protection for a work employing preexisting material in which copyright subsists does not extend to any part of the work in which such material has been used unlawfully.",
                    "children": [],
                    "end_date": None,
                    "node": "/us/usc/t17/s103/a",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/us/usc/t17/s103/a/",
                },
                {
                    "heading": "",
                    "content": "The copyright in a compilation or derivative work extends only to the material contributed by the author of such work, as distinguished from the preexisting material employed in the work, and does not imply any exclusive right in the preexisting material. The copyright in such work is independent of, and does not affect or enlarge the scope, duration, ownership, or subsistence of, any copyright protection in the preexisting material.",
                    "children": [],
                    "end_date": None,
                    "node": "/us/usc/t17/s103/b",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/us/usc/t17/s103/b/",
                },
            ],
            "end_date": None,
            "node": "/us/usc/t17/s103",
            "start_date": "2013-07-18",
            "url": "https://authorityspoke.com/api/v1/us/usc/t17/s103/",
            "parent": "https://authorityspoke.com/api/v1/us/usc/t17/",
        }
    },
    "/us/usc/t17/s410/c": {
        "2013-07-18": {
            "heading": "",
            "content": "In any judicial proceedings the certificate of a registration made before or within five years after first publication of the work shall constitute prima facie evidence of the validity of the copyright and of the facts stated in the certificate. The evidentiary weight to be accorded the certificate of a registration made thereafter shall be within the discretion of the court.",
            "children": [],
            "end_date": None,
            "node": "/us/usc/t17/s410/c",
            "start_date": "2013-07-18",
            "url": "https://authorityspoke.com/api/v1/us/usc/t17/s410/c/",
            "parent": "https://authorityspoke.com/api/v1/us/usc/t17/s410/",
        }
    },
    "/us/cfr/t37/s202.1": {
        "1992-02-21": {
            "node": "/us/cfr/t37/s202.1",
            "heading": "Material not subject to copyright.",
            "content": "The following are examples of works not subject to copyright and applications for registration of such works cannot be entertained:",
            "children": [
                {
                    "node": "/us/cfr/t37/s202.1/a",
                    "start_date": "1992-02-21",
                    "selection": True,
                    "heading": "",
                    "content": "Words and short phrases such as names, titles, and slogans; familiar symbols or designs; mere variations of typographic ornamentation, lettering or coloring; mere listing of ingredients or contents;",
                },
                {
                    "node": "/us/cfr/t37/s202.1/b",
                    "start_date": "1992-02-21",
                    "selection": True,
                    "heading": "",
                    "content": "Ideas, plans, methods, systems, or devices, as distinguished from the particular manner in which they are expressed or described in a writing;  ",
                },
                {
                    "node": "/us/cfr/t37/s202.1/c",
                    "start_date": "1992-02-21",
                    "selection": True,
                    "heading": "",
                    "content": "Blank forms, such as time cards, graph paper, account books, diaries, bank checks, scorecards, address books, report forms, order forms and the like, which are designed for recording information and do not in themselves convey information;",
                },
                {
                    "node": "/us/cfr/t37/s202.1/d",
                    "start_date": "1992-02-21",
                    "selection": True,
                    "heading": "",
                    "content": "Works consisting entirely of information that is common property containing no original authorship, such as, for example: Standard calendars, height and weight charts, tape measures and rulers, schedules of sporting events, and lists or tables taken from public documents or other common sources.",
                },
                {
                    "node": "/us/cfr/t37/s202.1/e",
                    "start_date": "1992-02-21",
                    "selection": True,
                    "heading": "",
                    "content": "Typeface as typeface.",
                },
            ],
            "start_date": "1992-02-21",
            "name": "short phrases copyright exception",
            "suffix": "familiar symbols or designs",
        }
    },
}
MOCK_BEARD_ACT_RESPONSES = {
    "/test/acts/47/1": {
        "1935-04-01": {
            "heading": "Short title",
            "content": "This Act may be cited as the Australian Beard Tax (Promotion of Enlightenment Values) Act 1934.",
            "children": [],
            "end_date": None,
            "node": "/test/acts/47/1",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/1/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/2": {
        "1935-04-01": {
            "heading": "Commencement",
            "content": "This Act shall commence on 1 April 1935.",
            "children": [],
            "end_date": None,
            "node": "/test/acts/47/2",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/2/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/3": {
        "1935-04-01": {
            "heading": "Purpose",
            "content": "",
            "children": [
                {
                    "heading": "",
                    "content": "This Act is enacted for the purpose of the promotion of enlightenment values.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/3/a",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/3/a/",
                }
            ],
            "end_date": None,
            "node": "/test/acts/47/3",
            "start_date": "2013-07-18",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/3/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/4": {
        "1935-04-01": {
            "heading": "Beard, defined",
            "content": "In this Act, beard means any facial hair no shorter than 5 millimetres in length that:",
            "children": [
                {
                    "heading": "",
                    "content": "occurs on or below the chin, or",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/4/a",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/4/a/",
                },
                {
                    "heading": "",
                    "content": "exists in an uninterrupted line from the front of one ear to the front of the other ear below the nose.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/4/b",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/4/b/",
                },
            ],
            "end_date": None,
            "node": "/test/acts/47/4",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/4/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/5": {
        "1935-04-01": {
            "heading": "Prohibition of beards",
            "content": "The wearing of any beard whatsoever, except as provided in section 6, is prohibited within the Commonwealth of Australia.",
            "children": [],
            "end_date": None,
            "node": "/test/acts/47/5",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/5/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/6": {
        "1935-04-01": {
            "heading": "Exemption",
            "content": "",
            "children": [
                {
                    "heading": "",
                    "content": "The office of the Department of Beards may, from time to time or as they see fit, grant exemptions to persons from the prohibition contained in section 5.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/6/1",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/6/1/",
                },
                {
                    "heading": "",
                    "content": "Any such exemption granted under subsection 1 is to be for no longer than a period of 12 months.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/6/2",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/6/2/",
                },
            ],
            "end_date": None,
            "node": "/test/acts/47/6",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/6/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/6A": {
        "1935-04-01": {
            "heading": "Levy of beard tax",
            "content": "Where the Department provides an exemption from the prohibition in section 5, except as defined in section 6D, the person to whom such exemption is granted shall be liable to pay to the Department of Beards such fee as may be levied under section 6B.",
            "children": [],
            "end_date": None,
            "node": "/test/acts/47/6A",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/6A/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/6B": {
        "1935-04-01": {
            "heading": "Regulatory power of the Minister for Beards",
            "content": "The Minister for Beards may, by Order in Council, issue such regulations, including but not limited to levies to be paid by persons exempted under section 6 from the prohibition in section 5, as is necessary for the good governance and financial stability of the Department of Beards.",
            "children": [],
            "end_date": None,
            "node": "/test/acts/47/6B",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/6B/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/6C": {
        "2013-07-18": {
            "heading": "Issuance of beardcoin",
            "content": "Where an exemption is granted under section 6, the Department of Beards shall issue to the person so exempted a token, hereinafter referred to as a beardcoin, that shall for all purposes be regarded as substantive proof of such exemption.",
            "children": [
                {
                    "heading": "",
                    "content": "The beardcoin shall be a cryptocurrency token using a blockchain approved by the Department of Beards.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/6C/1",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/6C/1/",
                },
                {
                    "heading": "",
                    "content": "The beardcoin's blockchain shall be in substantially the following form. WHEREAS this beardcoin is approved by the Department of Beards under section 6,\n                     it is secured by the following blockchain: □□□□□□□□□□.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/6C/1-con",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/6C/1-con/",
                },
            ],
            "end_date": None,
            "node": "/test/acts/47/6C",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/6C/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/6D": {
        "1935-04-01": {
            "heading": "Waiver of beard tax in special circumstances",
            "content": "",
            "children": [
                {
                    "heading": "",
                    "content": "The Department of Beards shall waive the collection of beard tax upon issuance of beardcoin under Section 6C where the reason the maintainer wears a beard is due to bona fide religious or cultural reasons.",
                    "children": [],
                    "end_date": "2013-07-18",
                    "node": "/test/acts/47/6D/1",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/6D/1@1935-04-01",
                },
                {
                    "heading": "",
                    "content": "The determination of the Department of Beards as to what constitutes bona fide religious or cultural reasons shall be final and no right of appeal shall exist.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/6D/2",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/6D/2@1935-04-01",
                },
            ],
            "end_date": None,
            "node": "/test/acts/47/6D",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/6D@1935-04-01",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47@1935-04-01",
        },
        "2013-07-18": {
            "heading": "Waiver of beard tax in special circumstances",
            "content": "",
            "children": [
                {
                    "heading": "",
                    "content": "The Department of Beards shall waive the collection of beard tax upon issuance of beardcoin under Section 6C where the reason the maintainer wears a beard is due to bona fide religious, cultural, or medical reasons.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/6D/1",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/6D/1/",
                },
                {
                    "heading": "",
                    "content": "The determination of the Department of Beards as to what constitutes bona fide religious or cultural reasons shall be final and no right of appeal shall exist.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/6D/2",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/6D/2/",
                },
            ],
            "end_date": None,
            "node": "/test/acts/47/6D",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/6D/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        },
    },
    "/test/acts/47/7": {
        "1935-04-01": {
            "heading": "Wearing of a beard without exemption",
            "content": "Any person found to be wearing a beard within the Commonwealth of Australia without proper exemption as granted under section 6 commits an offence.",
            "children": [],
            "end_date": None,
            "node": "/test/acts/47/7",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/7/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/7A": {
        "1935-04-01": {
            "heading": "Improper transfer of beardcoin",
            "content": "It shall be an offence to buy, sell, lend, lease, gift, transfer or receive in any way a beardcoin from any person or body other than the Department of Beards, except as provided in Part 4.",
            "children": [],
            "end_date": None,
            "node": "/test/acts/47/7A",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/7A/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/7B": {
        "1935-04-01": {
            "heading": "Counterfeit beardcoin",
            "content": "",
            "children": [
                {
                    "heading": "",
                    "content": "It shall be an offense to produce, alter, or manufacture tokens with the appearance of and purporting to be genuine beardcoin.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/7B/1",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/7B/1/",
                },
                {
                    "heading": "",
                    "content": "It shall be no defense to a charge under section 7A that the purchase, sale, lease, gift, transfer or receipt was of counterfeit beardcoin rather than genuine beardcoin.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/7B/2",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/7B/2/",
                },
            ],
            "end_date": None,
            "node": "/test/acts/47/7B",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/7B/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/8": {
        "1935-04-01": {
            "heading": "Notice to remedy",
            "content": "",
            "children": [
                {
                    "heading": "",
                    "content": "Where an officer of the Department of Beards, Australian Federal Police, state or territorial police, or military police of the Australian Defence Force finds a person to be wearing a beard within the territory of the Commonwealth of Australia, and that person fails or is unable to produce a beardcoin as proof of holding an exemption under section 6, that officer shall in the first instance issue such person a notice to remedy.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/8/1",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/8/1@1935-04-01",
                },
                {
                    "heading": "",
                    "content": "Any such person issued a notice to remedy under subsection 1 must either:",
                    "children": [
                        {
                            "heading": "",
                            "content": "shave in such a way that they are no longer in breach of section 5, or",
                            "children": [],
                            "end_date": None,
                            "node": "/test/acts/47/8/2/a",
                            "start_date": "1935-04-01",
                            "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2/a@1935-04-01",
                        },
                        {
                            "heading": "",
                            "content": "obtain a beardcoin from the Department of Beards",
                            "children": [],
                            "end_date": "2013-07-18",
                            "node": "/test/acts/47/8/2/b",
                            "start_date": "1935-04-01",
                            "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2/b@1935-04-01",
                        },
                        {
                            "heading": "",
                            "content": "within 14 days of such notice being issued to them.",
                            "children": [],
                            "end_date": "2013-07-18",
                            "node": "/test/acts/47/8/2/b-con",
                            "start_date": "1935-04-01",
                            "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2/b-con@1935-04-01",
                        },
                    ],
                    "end_date": None,
                    "node": "/test/acts/47/8/2",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2@1935-04-01",
                },
            ],
            "end_date": None,
            "node": "/test/acts/47/8",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/8@1935-04-01",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47@1935-04-01",
        },
        "2013-07-18": {
            "heading": "Notice to remedy",
            "content": "",
            "children": [
                {
                    "heading": "",
                    "content": "Where an officer of the Department of Beards, Australian Federal Police, state or territorial police, or military police of the Australian Defence Force finds a person to be wearing a beard within the territory of the Commonwealth of Australia, and that person fails or is unable to produce a beardcoin as proof of holding an exemption under section 6, that officer shall in the first instance issue such person a notice to remedy.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/8/1",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/8/1/",
                },
                {
                    "heading": "",
                    "content": "Any such person issued a notice to remedy under subsection 1 must either:",
                    "children": [
                        {
                            "heading": "",
                            "content": "shave in such a way that they are no longer in breach of section 5, or",
                            "children": [],
                            "end_date": None,
                            "node": "/test/acts/47/8/2/a",
                            "start_date": "1935-04-01",
                            "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2/a/",
                        },
                        {
                            "heading": "",
                            "content": "remove the beard with electrolysis, or",
                            "children": [],
                            "end_date": None,
                            "node": "/test/acts/47/8/2/b",
                            "start_date": "2013-07-18",
                            "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2/b/",
                        },
                        {
                            "heading": "",
                            "content": "",
                            "children": [],
                            "end_date": None,
                            "node": "/test/acts/47/8/2/b-con",
                            "start_date": "2013-07-18",
                            "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2/b-con/",
                        },
                        {
                            "heading": "",
                            "content": "remove the beard with a laser, or",
                            "children": [],
                            "end_date": None,
                            "node": "/test/acts/47/8/2/c",
                            "start_date": "2013-07-18",
                            "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2/c/",
                        },
                        {
                            "heading": "",
                            "content": "obtain a beardcoin from the Department of Beards",
                            "children": [],
                            "end_date": None,
                            "node": "/test/acts/47/8/2/d",
                            "start_date": "2013-07-18",
                            "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2/d/",
                        },
                        {
                            "heading": "",
                            "content": "within 14 days of such notice being issued to them.",
                            "children": [],
                            "end_date": None,
                            "node": "/test/acts/47/8/2/d-con",
                            "start_date": "2013-07-18",
                            "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2/d-con/",
                        },
                    ],
                    "end_date": None,
                    "node": "/test/acts/47/8/2",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/8/2/",
                },
            ],
            "end_date": None,
            "node": "/test/acts/47/8",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/8/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        },
    },
    "/test/acts/47/9": {
        "2013-07-18": {
            "heading": "Penalties",
            "content": "",
            "children": [
                {
                    "heading": "",
                    "content": "Any person summarily convicted of a first offence under section 7 of unlawfully wearing a beard within the Commonwealth of Australia shall be liable to a fine not exceeding $200.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/9/1",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/9/1/",
                },
                {
                    "heading": "",
                    "content": "Any person summarily convicted of a second or subsequent offence under section 7 shall be liable to a fine not exceeding $1000, or a period of imprisonment until such time as they no longer are in breach of section 5.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/9/2",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/9/2/",
                },
                {
                    "heading": "",
                    "content": "No penalty shall be applied to any person who, within 14 days of receiving a notice to remedy under section 8(1), takes the action required of them under section 8(2).",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/9/3",
                    "start_date": "1935-04-01",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/9/3/",
                },
                {
                    "heading": "",
                    "content": "",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/9/3-con",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/9/3-con/",
                },
                {
                    "heading": "",
                    "content": "Any person convicted of an offence under section 7A or section 7B(1) shall be liable to a fine not exceeding $5000 and/or a period of imprisonment not exceeding 12 months.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/9/4",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/9/4/",
                },
            ],
            "end_date": None,
            "node": "/test/acts/47/9",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/9/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/10": {
        "1935-04-01": {
            "heading": "Purpose of Part 4.—",
            "content": "Part 4 of the Australian Beard Tax (Promotion of Enlightement Values) Act 1934 exists for the purpose of incentivising Australians, and those visiting Australia, to relieve themselves from the burdensome habit of wearing a beard.",
            "children": [],
            "end_date": None,
            "node": "/test/acts/47/10",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/10/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/11": {
        "1935-04-01": {
            "heading": "Licensed repurchasers of beardcoin",
            "content": "The Department of Beards may issue licenses to such barbers, hairdressers or other male grooming professionals as they see fit to purchase a beardcoin from a customer whose beard they have removed, and to resell those beardcoins to the Department of Beards.",
            "children": [],
            "end_date": "2013-07-18",
            "node": "/test/acts/47/11",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/11@1935-04-01",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47@1935-04-01",
        },
        "2013-07-18": {
            "heading": "Licensed repurchasers of beardcoin",
            "content": "The Department of Beards may issue licenses to such",
            "children": [
                {
                    "heading": "",
                    "content": "barbers,",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/11/i",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/11/i/",
                },
                {
                    "heading": "",
                    "content": "hairdressers, or",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/11/ii",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/11/ii/",
                },
                {
                    "heading": "",
                    "content": "other male grooming professionals",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/11/iii",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/11/iii/",
                },
                {
                    "heading": "",
                    "content": "as they see fit to purchase a beardcoin from a customer",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/11/iii-con",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/11/iii-con/",
                },
                {
                    "heading": "",
                    "content": "whose beard they have removed,",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/11/iv",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/11/iv/",
                },
                {
                    "heading": "",
                    "content": "and to resell those beardcoins to the Department of Beards.",
                    "children": [],
                    "end_date": None,
                    "node": "/test/acts/47/11/iv-con",
                    "start_date": "2013-07-18",
                    "url": "https://authorityspoke.com/api/v1/test/acts/47/11/iv-con/",
                },
            ],
            "end_date": None,
            "node": "/test/acts/47/11",
            "start_date": "2013-07-18",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/11/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        },
    },
    "/test/acts/47/12": {
        "1935-04-01": {
            "heading": "Rate to be paid to repurchasers of beardcoin",
            "content": "The value to be transfered to licensed repurchasers of beardcoin under section 11 shall be defined by Order in Council by the Minister for Beards under section 6B on a per coin basis.",
            "children": [],
            "end_date": None,
            "node": "/test/acts/47/12",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/12/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        }
    },
    "/test/acts/47/13": {
        "1935-04-01": {
            "heading": "Removal of GST from razors and shavers",
            "content": "The items set out in Annexe 1 of this Act shall be exempt from the goods and sales tax provisions of the Taxation Administration Act 1953.",
            "children": [],
            "end_date": "2013-07-18",
            "node": "/test/acts/47/13",
            "start_date": "1935-04-01",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/13@2000-01-01",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47@2000-01-01",
        },
        "2013-07-18": {
            "heading": "",
            "content": "",
            "children": [],
            "end_date": None,
            "node": "/test/acts/47/13",
            "start_date": "2013-07-18",
            "url": "https://authorityspoke.com/api/v1/test/acts/47/13/",
            "parent": "https://authorityspoke.com/api/v1/test/acts/47/",
        },
    },
}

MOCK_USC_CLIENT = JSONRepository(responses=MOCK_USC_RESPONSES)
MOCK_BEARD_ACT_CLIENT = JSONRepository(responses=MOCK_BEARD_ACT_RESPONSES)
